/****************************************************************************
* Company:    ICPDAS
* Author:     Mac 
* Created:    2007 11 30  
* Notes:      This was created to communicate with your WinPAC Serial COM Port  
***************************************************************************/ 

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.Net.Sockets;
using Microsoft.Win32;
using WinPacNet;

namespace SendToCOM
{
    public partial class frmSendToCOM : Form
    {
        //================================================
        //            2007.12.14 Mac completed the following functions
        //1.Send data for using string mode
        //2.Send data for using binary mode
        //3.Can add CRC for binary mode
        //4.There are two modes which are polling and timer mode to send data automatically.
        //5.Display start and end time(for polling/timer)
        //6.To calculate and analyze sended packet and received packet and display the result.
        //
        //           2010.3.8 Tunglu Added                  
        //7.Support new COM port name for COM10,COM11,....
        //  Old COM port name is MSA1~MSA8,MSB~MSB8 and etc for COM10,COM11...
        //8.Add "Slot" combobox to support COM0 for 87K modules
        //================================================
        private static string Version = "V1.1.0  2011/10/19";
        
        SerialPort comPort = new SerialPort();
        private long lCmdPacketQuantity = 0;
        private long lCmdTotal = 0;
        private long lRevPacketSize = 0;
        private long lRevPacketQuantity = 0;
        private long lRevTotal = 0;
        private string endChr = "";
        private string sCmd="";
        private byte[] bCmd;
        private string sCRC = "";
        private byte[] bCRC = new byte[2];
        private bool bPolling = false;
        private readonly static ushort[] crcTable = {
        	0X0000, 0XC0C1, 0XC181, 0X0140, 0XC301, 0X03C0, 0X0280, 0XC241,
        	0XC601, 0X06C0, 0X0780, 0XC741, 0X0500, 0XC5C1, 0XC481, 0X0440,
        	0XCC01, 0X0CC0, 0X0D80, 0XCD41, 0X0F00, 0XCFC1, 0XCE81, 0X0E40,
        	0X0A00, 0XCAC1, 0XCB81, 0X0B40, 0XC901, 0X09C0, 0X0880, 0XC841,
        	0XD801, 0X18C0, 0X1980, 0XD941, 0X1B00, 0XDBC1, 0XDA81, 0X1A40,
        	0X1E00, 0XDEC1, 0XDF81, 0X1F40, 0XDD01, 0X1DC0, 0X1C80, 0XDC41,
        	0X1400, 0XD4C1, 0XD581, 0X1540, 0XD701, 0X17C0, 0X1680, 0XD641,
        	0XD201, 0X12C0, 0X1380, 0XD341, 0X1100, 0XD1C1, 0XD081, 0X1040,
        	0XF001, 0X30C0, 0X3180, 0XF141, 0X3300, 0XF3C1, 0XF281, 0X3240,
        	0X3600, 0XF6C1, 0XF781, 0X3740, 0XF501, 0X35C0, 0X3480, 0XF441,
        	0X3C00, 0XFCC1, 0XFD81, 0X3D40, 0XFF01, 0X3FC0, 0X3E80, 0XFE41,
        	0XFA01, 0X3AC0, 0X3B80, 0XFB41, 0X3900, 0XF9C1, 0XF881, 0X3840,
        	0X2800, 0XE8C1, 0XE981, 0X2940, 0XEB01, 0X2BC0, 0X2A80, 0XEA41,
        	0XEE01, 0X2EC0, 0X2F80, 0XEF41, 0X2D00, 0XEDC1, 0XEC81, 0X2C40,
        	0XE401, 0X24C0, 0X2580, 0XE541, 0X2700, 0XE7C1, 0XE681, 0X2640,
        	0X2200, 0XE2C1, 0XE381, 0X2340, 0XE101, 0X21C0, 0X2080, 0XE041,
        	0XA001, 0X60C0, 0X6180, 0XA141, 0X6300, 0XA3C1, 0XA281, 0X6240,
        	0X6600, 0XA6C1, 0XA781, 0X6740, 0XA501, 0X65C0, 0X6480, 0XA441,
        	0X6C00, 0XACC1, 0XAD81, 0X6D40, 0XAF01, 0X6FC0, 0X6E80, 0XAE41,
        	0XAA01, 0X6AC0, 0X6B80, 0XAB41, 0X6900, 0XA9C1, 0XA881, 0X6840,
        	0X7800, 0XB8C1, 0XB981, 0X7940, 0XBB01, 0X7BC0, 0X7A80, 0XBA41,
        	0XBE01, 0X7EC0, 0X7F80, 0XBF41, 0X7D00, 0XBDC1, 0XBC81, 0X7C40,
        	0XB401, 0X74C0, 0X7580, 0XB541, 0X7700, 0XB7C1, 0XB681, 0X7640,
        	0X7200, 0XB2C1, 0XB381, 0X7340, 0XB101, 0X71C0, 0X7080, 0XB041,
        	0X5000, 0X90C1, 0X9181, 0X5140, 0X9301, 0X53C0, 0X5280, 0X9241,
        	0X9601, 0X56C0, 0X5780, 0X9741, 0X5500, 0X95C1, 0X9481, 0X5440,
        	0X9C01, 0X5CC0, 0X5D80, 0X9D41, 0X5F00, 0X9FC1, 0X9E81, 0X5E40,
        	0X5A00, 0X9AC1, 0X9B81, 0X5B40, 0X9901, 0X59C0, 0X5880, 0X9841,
        	0X8801, 0X48C0, 0X4980, 0X8941, 0X4B00, 0X8BC1, 0X8A81, 0X4A40,
        	0X4E00, 0X8EC1, 0X8F81, 0X4F40, 0X8D01, 0X4DC0, 0X4C80, 0X8C41,
        	0X4400, 0X84C1, 0X8581, 0X4540, 0X8701, 0X47C0, 0X4680, 0X8641,
        	0X8201, 0X42C0, 0X4380, 0X8341, 0X4100, 0X81C1, 0X8081, 0X4040 
        };
        public frmSendToCOM()
        {
            InitializeComponent();
        }
        private string HardwareID()
        {
            //MQ200/C270W
            RegistryKey rkey = Registry.LocalMachine;
            RegistryKey icpdas = rkey.OpenSubKey("Drivers\\Display\\MQ200", true);
            RegistryKey icpdas1 = rkey.OpenSubKey("HARDWARE\\MODULETYPE\\", true);
            if (icpdas!=null)
            {
                return "MQ200";
            }
            else if (icpdas1!=null)
            {
                return icpdas1.GetValue("HWID").ToString();
            }
            else return "C270W";
        }
        private void frmSendToCOM_Load(object sender, EventArgs e)
        {
            int i;
            cmbBaud.SelectedIndex = 7;
            cmbDataBit.SelectedIndex = 1;
            cmbParity.SelectedIndex = 0;
            cmbStopBit.SelectedIndex = 0;
            rbNone.Checked = true;
            comPort.DataReceived += new SerialDataReceivedEventHandler(port_DataReceived);
            //int counter = 10; //if com port number >10, change com port name
            foreach (string s in SerialPort.GetPortNames())
            {
                //Console.WriteLine(s);

                if (s.Length == 4)
                {
                    //COM0~COM9
                    cmbPort.Items.Add(s);
                }
                else 
                {
                    /*
                     COM10 and later
                     New com port name:
                           "$device\\COM10" ,
                           "$device\\COM11" ,
                           "$device\\COM12" ,
                           "$device\\COM13" ,                                                      
                           .................                                                                                            
                     */
                    string ss;
                    //           2011.10.19 Modified by Sean
                    //1.Get the actual COM port name from the SerialPort component
                    //=============================================================
                    ss = s.Substring(8);// "COM" +counter.ToString();
                    //counter++;
                    cmbPort.Items.Add(ss);
                }
            }
            cmbPort.SelectedIndex = 0;

            //Old com port name for COM10,COM11,COM12.....
            for(i=1;i<9;i++)
                cmbPort.Items.Add(Convert.ToString("MSA"+i));            
            for (i = 1; i < 9; i++)
                cmbPort.Items.Add(Convert.ToString("MSB" + i));
            for (i = 1; i < 9; i++)
                cmbPort.Items.Add(Convert.ToString("MSC" + i));
            for (i = 1; i < 9; i++)
                cmbPort.Items.Add(Convert.ToString("MSD" + i)); 

            //Show version
            this.Text = "ICPDAS Send to COM " + Version;

            //Adjust screen size for different platform
            if (HardwareID()=="VP35")
            {
                this.Width = (System.Windows.Forms.Screen.PrimaryScreen.WorkingArea.Width ) * 95 / 100;
                this.Height = (System.Windows.Forms.Screen.PrimaryScreen.WorkingArea.Height ) * 95 / 100;
            }
            else if (HardwareID() == "MQ200")
            {
                switch (GetReg("HKEY_LOCAL_MACHINE\\Drivers\\Display\\MQ200\\CxScreen"))
                {
                    case 320:
                        this.Height = (System.Windows.Forms.Screen.PrimaryScreen.WorkingArea.Height) * 95 / 100;
                        this.Width = (System.Windows.Forms.Screen.PrimaryScreen.WorkingArea.Width) * 95 / 100;
                        break;
                }
            }
           }
       
        uint GetReg(string KeyName)
        {
            uint Val = 0;
            Val = WinPAC.pac_RegGetDWORD(KeyName);
            return Val;
        }

        private void btTimerStart_Click(object sender, EventArgs e)
        {
            if (rdDataBinary.Checked)
            {
                bCmd = HexStringToByteArray(txtMsg.Text);
                sCmd = ByteArrayToHexString(bCmd);
                if (chkCRC.Checked)
                {
                    bCRC = CalculateCrc(bCmd);
                    sCRC = ByteArrayToHexString(bCRC);
                    bCmd = combineArrays(bCmd, bCRC);
                    sCmd = sCmd + sCRC;
                }
                txtCmdSize.Text = bCmd.Length.ToString();
            }
            else
            {
                sCmd = txtMsg.Text + endChr;
                txtCmdSize.Text = sCmd.Length.ToString();
            }
            
            btTimerStop.Enabled = true;
            btTimerStart.Enabled = false;
            btPolling.Enabled = false;
            btPollingStop.Enabled = false;
            rdDataBinary.Enabled = false;
            rdDataString.Enabled = false;
            panelEndChar.Enabled = false;
            txtStartTime.Text = System.DateTime.Now.ToString();
            txtStopTime.Text = "Stop Time";
            timerSend.Enabled = true;
        }
        private void btTimerStop_Click(object sender, EventArgs e)
        {
            timerSend.Enabled = false;
            btTimerStart.Enabled = true;
            btTimerStop.Enabled = false;
            btPolling.Enabled = true;
            btPollingStop.Enabled = true;
            rdDataBinary.Enabled = true;
            rdDataString.Enabled = true;
            panelEndChar.Enabled = true;
            txtStopTime.Text = System.DateTime.Now.ToString();
        }
        private void btOpenCOM_Click(object sender, EventArgs e)
        {
            comPort.BaudRate = int.Parse(cmbBaud.Text);
            comPort.DataBits = int.Parse(cmbDataBit.Text);
            if (cmbParity.Text.Substring(0, 1) == "0")
            {
                comPort.Parity = Parity.None;
            }
            else if (cmbParity.Text.Substring(0, 1) == "1")
            {
                comPort.Parity = Parity.Odd;
            }
            else if (cmbParity.Text.Substring(0, 1) == "2")
            {
                comPort.Parity = Parity.Even;
            }
            if (cmbStopBit.Text == "0")
            {
                comPort.StopBits = StopBits.None;
            }
            else if (cmbStopBit.Text == "1")
            {
                comPort.StopBits = StopBits.One;
            }
            

            int ComPortNumber = Int32.Parse(cmbPort.Text.Substring(3));
            if (ComPortNumber > 9)
            {
                comPort.PortName = "$device\\" + cmbPort .Text ;
            }
            else if (ComPortNumber == 0)
            {
                labSlot.Enabled = true;
                cmbSlot.Enabled = true;
                cmbSlot.SelectedIndex = 0;
                txtMsg.Text = "$00M";
            }
            else
            {
                comPort.PortName = cmbPort.Text;
            }

            try
            {
                if (ComPortNumber != 0)
                {
                    comPort.Open();                    
                }
                else 
                {
                    //Open COM0 using WinPacNet.dll
                    hPort_COM0 = WinPAC.uart_Open("COM0:");                    
                }
                btCloseCOM.Enabled = true;
                btOpenCOM.Enabled = false;
                btSend.Enabled = true;
            }
            catch (Exception ex) 
            {
                MessageBox.Show(ex.Message+"Or had been opened.");
            }

            btTimerStart.Enabled = true;
            btSend.Enabled = true;
            btPolling.Enabled = true;
            btPollingStop.Enabled = true;
        }
        private void btCloseCOM_Click(object sender, EventArgs e)
        {
            timerSend.Enabled = false;
            btOpenCOM.Enabled = true;
            btCloseCOM.Enabled = false;
            btSend.Enabled = false;
            comPort.Close();
            btTimerStart.Enabled = false;
            btSend.Enabled = false;
            btPolling.Enabled = false;
            btPollingStop.Enabled = false;
            
            if(cmbSlot.Enabled)
            {
                WinPAC.uart_Close(hPort_COM0);//close com0
            }
        }
        private void btClearSend_Click(object sender, EventArgs e)
        {
            txtSend.Text = "";
        }
        private void btClearReceive_Click(object sender, EventArgs e)
        {
            txtReceive.Text = "";
            txtSend.Text = "";
        }
        private void btSetTimer_Click(object sender, EventArgs e)
        {
            timerSend.Interval = Int16.Parse(txtTimeInterval.Text);
        }
        private void btPolling_Click(object sender, EventArgs e)
        {
            btPollingStop.Visible = true;
            btPolling.Visible = false;
            bPolling = true;
            if (rdDataBinary.Checked)
            {
                bCmd = HexStringToByteArray(txtMsg.Text);
                sCmd = ByteArrayToHexString(bCmd);
                if (chkCRC.Checked)
                {
                    bCRC = CalculateCrc(bCmd);
                    sCRC = ByteArrayToHexString(bCRC);
                    bCmd = combineArrays(bCmd, bCRC);
                    sCmd = sCmd + sCRC;
                }
                txtCmdSize.Text = bCmd.Length.ToString();
            }
            else
            {
                sCmd = txtMsg.Text + endChr;
                txtCmdSize.Text = sCmd.Length.ToString();
            }
            btSend.Enabled = false;
            btTimerStart.Enabled = false;
            panelEndChar.Enabled = false;
            txtStartTime.Text = System.DateTime.Now.ToString();
            txtStopTime.Text = "Stop Time";
            while (bPolling)
            {
                lCmdPacketQuantity++;
                if (rdDataBinary.Checked)
                {
                    SendByteToCOM(bCmd, sCmd);
                }
                else
                {
                    SendStringToCOM(sCmd);
                }
                Application.DoEvents();
            }
        }
        private void btClearAnalysis_Click(object sender, EventArgs e)
        {           
            lCmdPacketQuantity = 0;
            lCmdTotal = 0;
            lRevPacketSize = 0;
            lRevPacketQuantity = 0;
            lRevTotal = 0;           
            txtCmdTotal.Text = "0";
            txtCmdTotalPacket.Text = "0";
            txtRevSize.Text = "0";
            txtRevTotal.Text = "0";
            txtRevTotalPacket.Text = "0";
        }

        /// <RadioButton>
        /// 
        /// </RadioButton>
        private void rbLFCR_CheckedChanged(object sender, EventArgs e)
        {
            endChr = "\n\r";
        }
        private void rbCR_CheckedChanged(object sender, EventArgs e)
        {
            endChr = "\r";
        }
        private void rbCRLF_CheckedChanged(object sender, EventArgs e)
        {
            endChr = "\r\n";
        }
        private void rbLF_CheckedChanged(object sender, EventArgs e)
        {
            endChr = "\n";
        }
        private void rbNone_CheckedChanged(object sender, EventArgs e)
        {
            endChr = "";
        }
        private void rdDataString_CheckedChanged(object sender, EventArgs e)
        {
            txtMsg.Text = "string";
            panelEndChar.Enabled = true;
            chkCRC.Enabled = false;
        }
        private void rdDataBinary_CheckedChanged(object sender, EventArgs e)
        {
            txtMsg.Text = "01 04 00 00 00 01";
            panelEndChar.Enabled = false;
            chkCRC.Enabled = true;
        }

        /// <TextBox>
        /// 
        /// </TextBox>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtSend_TextChanged(object sender, EventArgs e)
        {
            if (txtSend.Text.Length > 3000)
                txtSend.Text = "";
        }
        private void txtReceive_TextChanged(object sender, EventArgs e)
        {
            if (txtReceive.Text.Length > 3000)
                txtReceive.Text = "";
        }


        /// <Polling>
        /// Polling mode
        /// </Polling>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btPollingStop_Click(object sender, EventArgs e)
        {
            btPollingStop.Visible = false;
            btPolling.Visible = true;
            bPolling = false;
            btSend.Enabled = true;
            btTimerStart.Enabled = true;
            panelEndChar.Enabled = true;
            txtStopTime.Text = System.DateTime.Now.ToString();
        }
        /// <Timer>
        /// Timer mode
        /// </Timer>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timerSend_Tick(object sender, EventArgs e)
        {
            lCmdPacketQuantity++;
            if (rdDataBinary.Checked)
            {
                SendByteToCOM(bCmd, sCmd);
            }
            else
            {
                SendStringToCOM(sCmd);
            }
        }
        /// <Send>
        /// 
        /// </Send>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// 
        IntPtr hPort_COM0;
        private void btSend_Click(object sender, EventArgs e)
        {
            if (labSlot.Enabled == true)//if open com0 add the parameters of slot
            {                
                //Send command and receive response
                byte[] response = new byte[60];
                WinPAC.uart_SendCmd(hPort_COM0, WinPAC.AnsiString(txtMsg.Text), response);
                //Show response
                txtReceive.Text = WinPAC.WideString(response);
            }

            lCmdPacketQuantity++;
            if (rdDataBinary.Checked)
            {
                bCmd = HexStringToByteArray(txtMsg.Text);
                sCmd = ByteArrayToHexString(bCmd);
                if (chkCRC.Checked)
                {
                    bCRC = CalculateCrc(bCmd);
                    sCRC = ByteArrayToHexString(bCRC);
                    bCmd = combineArrays(bCmd, bCRC);
                    sCmd = sCmd + sCRC;
                }
                if (labSlot.Enabled != true)
                {
                    SendByteToCOM(bCmd, sCmd);
                }
                txtCmdSize.Text = bCmd.Length.ToString();
            }
            else
            {
                sCmd = txtMsg.Text + endChr;
                if (labSlot.Enabled != true)
                {
                    SendStringToCOM(sCmd);
                }
                txtCmdSize.Text = sCmd.Length.ToString();
            }
        }


        /// <SendStringToCOM>
        /// send String
        /// </SendStringToCOM>
        /// <param name="sData"></param>
        private void SendStringToCOM(string sData)
        {
            comPort.Write(sData);
            txtSend.Text = txtSend.Text + sData;
            txtCmdTotalPacket.Text = lCmdPacketQuantity.ToString();
            lCmdTotal = lCmdTotal + sData.Length;
            txtCmdTotal.Text = lCmdTotal.ToString();
        }
        /// <SendByteToCOM>
        /// send Binary
        /// Modbus/RTU will send CRC,receive have too
        /// </SendByteToCOM>
        /// <param name="bData"></param>
        /// <param name="sData"></param>
        private void SendByteToCOM(byte[] bData, string sData)
        {
            comPort.Write(bData, 0, bData.Length);
            txtSend.Text = txtSend.Text + sData;
            txtCmdTotalPacket.Text = lCmdPacketQuantity.ToString();
            lCmdTotal = lCmdTotal + bData.Length;
            txtCmdTotal.Text = lCmdTotal.ToString();
        }

        /// <PortReceive>
        /// 
        /// </PortReceive>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void port_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            this.Invoke(new EventHandler(AddReceive));

        }
        private void AddReceive(object s, EventArgs e)
        {
            lRevPacketQuantity++;
            if (rdDataBinary.Checked)
            {
                // Obtain the number of bytes waiting in the port's buffer
                int bytes = comPort.BytesToRead;
                // Create a byte array buffer to hold the incoming data
                byte[] buffer = new byte[bytes];                
                // Read the data from the port and store it in our buffer
                comPort.Read(buffer, 0, bytes);                
                txtReceive.Text = txtReceive.Text + ByteArrayToHexString(buffer);               
                lRevPacketSize = bytes;
                lRevTotal = lRevTotal + lRevPacketSize;
            }
            else
            {
                string sTemp;
                lRevPacketSize = comPort.BytesToRead;
                sTemp = comPort.ReadExisting();               
                txtReceive.Text = txtReceive.Text + sTemp;
                lRevTotal = lRevTotal + lRevPacketSize;
            }
            txtRevSize.Text = lRevPacketSize.ToString();
            txtRevTotal.Text = lRevTotal.ToString();
            txtRevTotalPacket.Text = lRevPacketQuantity.ToString();
        }


        /// <SubFunction>
        /// 
        /// </SubFunction>
        /// <param name="bCmd"></param>
        /// <param name="bCRC"></param>
        /// <returns></returns>
        private byte[] combineArrays(byte[] bCmd, byte[] bCRC)
        {
            byte[] combArray = new byte[bCmd.Length + bCRC.Length];
            bCmd.CopyTo(combArray, 0);
            bCRC.CopyTo(combArray, bCmd.Length);
            return combArray;
        }
        private byte[] CalculateCrc(byte[] data)
        {
            if (data == null)
                throw new ArgumentNullException("data");

            ushort crc = ushort.MaxValue;

            foreach (byte b in data)
            {
                byte tableIndex = (byte)(crc ^ b);
                crc >>= 8;
                crc ^= crcTable[tableIndex];
            }

            return BitConverter.GetBytes(crc);
        }
        private byte[] HexStringToByteArray(string s)
        {
            s = s.Replace(" ", "");
            byte[] buffer = new byte[s.Length / 2];
            for (int i = 0; i < s.Length; i += 2)
                buffer[i / 2] = (byte)Convert.ToByte(s.Substring(i, 2), 16);
            return buffer;
        }
        private string ByteArrayToHexString(byte[] data)
        {
            StringBuilder sb = new StringBuilder(data.Length * 3);
            foreach (byte b in data)
                sb.Append(Convert.ToString(b, 16).PadLeft(2, '0').PadRight(3, ' '));
            return sb.ToString().ToUpper();
        }

        private void cmbSlot_SelectedIndexChanged(object sender, EventArgs e)
        {
            WinPAC.pac_ChangeSlot(Convert.ToByte(cmbSlot.Text));//change slot
        }

        private void cmbPort_SelectedIndexChanged(object sender, EventArgs e)
        {
            labSlot.Enabled = false;
            cmbSlot.Enabled = false;
            txtMsg.Text = "string";            
        }

  

    }
}