﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace _87k_pwm
{
    public partial class Form1 : Form
    {
        private IntPtr hPort  ;
        private Int32 slotIndex = 7;
        private static byte[] pwmStartBit = new byte[8];
        private static bool activated = false;

        public Form1()
        {
            InitializeComponent();
            cboSlot.SelectedIndex = slotIndex;
        }

        private void ConfigPWMSettings(object sender, EventArgs e)
        {
            TextBox[] hzArr = new TextBox[] { txtHz0, txtHz1, txtHz2, txtHz3, txtHz4, txtHz5, txtHz6, txtHz7 };
            TextBox[] dutyArr = new TextBox[] { txtDuty0, txtDuty1, txtDuty2, txtDuty3, txtDuty4, txtDuty5, txtDuty6, txtDuty7 };
            ComboBox[] modeArr = new ComboBox[] { cboMode0, cboMode1, cboMode2, cboMode3, cboMode4, cboMode5, cboMode6, cboMode7 };
            TextBox[] cntArr = new TextBox[] { txtCnt0, txtCnt1, txtCnt2, txtCnt3, txtCnt4, txtCnt5, txtCnt6, txtCnt7 };

            Int16 chIndex = 0;
            chIndex = Int16.Parse(((Button)sender).Tag.ToString());
            bool ret = false ;

            hPort = PACNET.UART.Open("");

            ret = PACNET.PWM.SetPWMFrequency(hPort, slotIndex, chIndex, Convert.ToUInt32(hzArr[chIndex].Text));

            if (!ret)
            {
                uint ec = PACNET.ErrHandling.GetLastError();
                MessageBox.Show(((PACNET.ErrCode)ec).ToString() + "\nError Code: 0x" + ec.ToString("X"));
            }
            else
            {
                ret = PACNET.PWM.SetPWMDuty(hPort, slotIndex, chIndex, Convert.ToSingle(dutyArr[chIndex].Text));
                ret = PACNET.PWM.SetPWMPulseCount(hPort, slotIndex, chIndex, Convert.ToInt32(cntArr[chIndex].Text));

                ret = PACNET.PWM.SetPWMMode(hPort, slotIndex, chIndex, (Int16)modeArr[chIndex].SelectedIndex);
               
                GetPWMConfig(chIndex);
            }

            PACNET.UART.Close(hPort);  
        }

        private void GetPWMConfig(Int16 chIndex)
        {
            TextBox[] hzArr = new TextBox[] { txtHz0, txtHz1, txtHz2, txtHz3, txtHz4, txtHz5, txtHz6, txtHz7 };
            TextBox[] dutyArr = new TextBox[] { txtDuty0, txtDuty1, txtDuty2, txtDuty3, txtDuty4, txtDuty5, txtDuty6, txtDuty7 };
            ComboBox[] modeArr = new ComboBox[] { cboMode0, cboMode1, cboMode2, cboMode3, cboMode4, cboMode5, cboMode6, cboMode7 };
            TextBox[] cntArr = new TextBox[] { txtCnt0, txtCnt1, txtCnt2, txtCnt3, txtCnt4, txtCnt5, txtCnt6, txtCnt7 };
            UInt32 freq = 0;

            bool ret = false;

            ret = PACNET.PWM.GetPWMFrequency(hPort, slotIndex, chIndex, ref freq);


            if (!ret)
            {
                uint ec = PACNET.ErrHandling.GetLastError();
                MessageBox.Show(((PACNET.ErrCode)ec).ToString() + "\nError Code: 0x" + ec.ToString("X"));
            }
            else
            {
                hzArr[chIndex].Text = freq.ToString();
                float duty = 0;
                ret = PACNET.PWM.GetPWMDuty(hPort, slotIndex, chIndex, ref duty);
                dutyArr[chIndex].Text = String.Format("{0:00.0}", duty);
                Int32 cnt = 0;
                ret = PACNET.PWM.GetPWMPulseCount(hPort, slotIndex, chIndex, ref cnt);
                cntArr[chIndex].Text = cnt.ToString();

                int mode = 0;
                ret = PACNET.PWM.GetPWMMode(hPort, slotIndex, chIndex, ref mode);
                modeArr[chIndex].SelectedIndex = mode;
            }
        }

        private void EnablePWM(object sender, EventArgs e)
        {
            if (activated)
            {
                Int16 chIndex = Int16.Parse(((CheckBox)sender).Tag.ToString());
                bool ret = false;
                
                CheckBox enBox = (CheckBox)sender;
                Int16 status = 0;
                if (enBox.Checked)
                {
                    pwmStartBit[chIndex] = 1;                
                }
                else
                {
                    pwmStartBit[chIndex] = 0;                  
                }
                
                for (int i = 0; i < 8; i++)
                    status += (Int16)(pwmStartBit[i] << i);

                hPort = PACNET.UART.Open("");
                ret = PACNET.PWM.SetPWMStart(hPort, slotIndex, status);
                if (!ret)
                {
                    uint ec = PACNET.ErrHandling.GetLastError();
                    MessageBox.Show(((PACNET.ErrCode)ec).ToString() + "\nError Code: 0x" + ec.ToString("X"));
                }
                PACNET.UART.Close(hPort);  
                
            }
        }

        private void btnSavePWM_Click(object sender, EventArgs e)
        {
            hPort = PACNET.UART.Open("");
            bool ret = PACNET.PWM.SavePWMConfig(hPort, slotIndex);
            if (!ret)
            {
                uint ec = PACNET.ErrHandling.GetLastError();
                MessageBox.Show(((PACNET.ErrCode)ec).ToString() + "\nError Code: 0x" + ec.ToString("X"));
            }
            PACNET.UART.Close(hPort);  
        }

        private void cboSlot_SelectedIndexChanged(object sender, EventArgs e)
        {
            slotIndex = cboSlot.SelectedIndex;
        }

        private void btnRead_Click(object sender, EventArgs e)
        {
            ReadPWM_DIStatus();
        }

        private void ReadPWM_DIStatus()
        {
            Label[] lbDI = { lbDI0, lbDI1, lbDI2, lbDI3, lbDI4, lbDI5, lbDI6, lbDI7 };
            Label[] lbPWM = { pwm0, pwm1, pwm2, pwm3, pwm4, pwm5, pwm6, pwm7 };
            CheckBox[] enPWM = { ckEnPWM0, ckEnPWM1, ckEnPWM2, ckEnPWM3, ckEnPWM4, ckEnPWM5, ckEnPWM6, ckEnPWM7 };

            byte[] pwmBit = new byte[8];
            byte[] diBit = new byte[8]; ;
            bool ret = false;

            hPort = PACNET.UART.Open("");
            ret = PACNET.PWM.GetPWMDIOStatus(hPort, slotIndex, pwmBit, diBit);
            PACNET.UART.Close(hPort);

            if (!ret)
            {
                uint ec = PACNET.ErrHandling.GetLastError();
                MessageBox.Show(((PACNET.ErrCode)ec).ToString() + "\nError Code: 0x" + ec.ToString("X"));
            }
            else
            {
                for (int i = 0; i < 8; i++)
                {
                    if (pwmBit[i] == 1)
                    {
                        lbPWM[i].Text = "ON";
                        lbPWM[i].BackColor = Color.Red;
                        activated = false;
                        enPWM[i].Checked = true;
                        activated = true;
                    }
                    else
                    {
                        lbPWM[i].Text = "OFF";
                        lbPWM[i].BackColor = Color.Gray;
                        activated = false;
                        enPWM[i].Checked = false;
                        activated = true;
                    }
                    if (diBit[i] == 1)
                    {
                        lbDI[i].Text = "ON";
                        lbDI[i].BackColor = Color.Green;
                    }
                    else
                    {
                        lbDI[i].Text = "OFF";
                        lbDI[i].BackColor = Color.Gray;
                    }
                }
            }
        }

        private void Form1_Activated(object sender, EventArgs e)
        {
            activated = true;
        }     
    }
}