﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Diagnostic
{
    public partial class Form1 : Form
    {
        const int startX = 20;
        const int startY = 30;
        const int UNIT_LENGTH = 50;
        const int GAP_SLOT = 5;
        const int GAP_MODULE_LABEL = 20;
        const int LABEL_HEIGHT = 20;

        int slotCount = 0;
        bool[] ModuleExist = new bool[8];
        string[] ModuleName = new string[8];
        Label[] lbSlot = new Label[8];
        Label[] lbmoduleName = new Label[8];

        Image headImage;  //image of CPU-power module
        Image moduleImage;  //image of I/O slot module
        Image emptyImage;  //image of a slot in empty status
        PictureBox pbHead = new PictureBox();
        PictureBox[] pbModule = new PictureBox[8];

        int iSlotStart = 0;

        public Form1()
        {
            InitializeComponent();

            if (Environment.OSVersion.Version.Major == 5)
            {
                iSlotStart = 0;
            }
            else
            {
                iSlotStart = 1;
            }

            headImage = imageList1.Images[1];
            moduleImage = imageList1.Images[2];
            emptyImage = imageList1.Images[0];

            //initialize image of CPU-power module
            pbHead.Image = headImage;
            pbHead.SizeMode = PictureBoxSizeMode.StretchImage;
            pbHead.Size = new Size(UNIT_LENGTH * 3, UNIT_LENGTH * 3);
            pbHead.Location = new Point(startX, startY);
            this.Controls.Add(pbHead);

            for (int i = 0; i <= 7 - iSlotStart; i++)
            {
                //initialize Labels of Slot1, Slot2, ...
                lbSlot[i] = new Label();
                lbSlot[i].Text = "Slot " + (i + iSlotStart).ToString();
                lbSlot[i].Size = new Size(UNIT_LENGTH, LABEL_HEIGHT);
                this.Controls.Add(lbSlot[i]);

                //initialize PictureBoxs which hold image of I/O module status
                pbModule[i] = new PictureBox();
                pbModule[i].SizeMode = PictureBoxSizeMode.StretchImage;
                pbModule[i].Size = new Size(UNIT_LENGTH, UNIT_LENGTH * 3);
                pbModule[i].Location = new Point(startX + UNIT_LENGTH * 3 + GAP_SLOT + (UNIT_LENGTH + GAP_SLOT) * i, startY);
                pbModule[i].Image = emptyImage;
                this.Controls.Add(pbModule[i]);

                //initialize Labels to hold module names
                lbmoduleName[i] = new Label();
                lbmoduleName[i].Size = new Size(UNIT_LENGTH, LABEL_HEIGHT);
                this.Controls.Add(lbmoduleName[i]);
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            slotCount = PACNET.Sys.GetSlotCount();

            if (Environment.OSVersion.Version.Major == 5)
            {
                slotCount -= 1;
            }

            printSlotLabels(startX + UNIT_LENGTH * 3 + GAP_SLOT,
                                       startY + UNIT_LENGTH * 3 + GAP_MODULE_LABEL);

            button1_Click(sender, e);
        }

        private void showModuleImages()
        {
            for (int i = iSlotStart; i <= 7; i++)
            {
                if (i <= slotCount)
                {
                    if (ModuleExist[i - iSlotStart])
                    {
                        pbModule[i - iSlotStart].Image = moduleImage;
                    }
                    else
                    {
                        pbModule[i - iSlotStart].Image = emptyImage;
                    }
                }
                else
                {
                    pbModule[i - iSlotStart].Visible = false;
                }
            }
        }

        private void printSlotLabels(int x, int y)
        {
            for (int i = iSlotStart; i <= 7; i++)
            {
                if (i <= slotCount)
                {
                    lbSlot[i - iSlotStart].Location = new Point(x + (UNIT_LENGTH + GAP_SLOT) * (i - iSlotStart), y);
                }
                else
                {
                    lbSlot[i - iSlotStart].Visible = false;
                }
            }
        }

        private void printModuleNameLabels(int x, int y)
        {
            for (int i = iSlotStart; i <= 7; i++)
            {
                if (i <= slotCount)
                {
                    lbmoduleName[i - iSlotStart].Location = new Point(x + (UNIT_LENGTH + GAP_SLOT) * (i - iSlotStart), y);
                }
                else
                {
                    lbmoduleName[i - iSlotStart].Visible = false;
                }
            }
        }

        private void button1_Click(object sender, EventArgs e)  //Diagnose
        {
            IntPtr h = PACNET.UART.Open("");  //COM1:,115200,N,8,1
            IntPtr _for_8K = IntPtr.Zero;// (IntPtr)0;    //used for pac_ModuleExists
            int moduleType = 0;
            byte[] cmd = new byte[64];
            byte[] result = new byte[64];

            cmd = PACNET.MISC.AnsiString("$00M");

            for (byte i = (byte)iSlotStart; i <= slotCount; i++)
            {
                ModuleExist[i - iSlotStart] = PACNET.Sys.ModuleExists(_for_8K, i);  //for 8K IO module only
                if (ModuleExist[i - iSlotStart])
                {
                    moduleType = PACNET.Sys.GetModuleName(i, ref ModuleName[i - iSlotStart]);
                    if (255 == moduleType)  //87K IO modules
                    {
                        PACNET.Sys.ChangeSlot(i);

                        bool iRet = PACNET.UART.SendCmd(h, cmd, result);  // $00M: ask the device name

                        if (!iRet)
                        {
                            uint ec = PACNET.ErrHandling.GetLastError();
                            MessageBox.Show(((PACNET.ErrCode)ec).ToString() + "\nError Code: 0x" + ec.ToString("X"));
                        }

                        string strResult = PACNET.MISC.WideString(result);
                        ModuleName[i - iSlotStart] = strResult.Substring(3);  //return format for example: !0087055
                    }
                }
                else
                {
                    ModuleName[i - iSlotStart] = "";
                }

                lbmoduleName[i - iSlotStart].Text = ModuleName[i - iSlotStart];
            }

            PACNET.UART.Close(h);

            printModuleNameLabels(startX + UNIT_LENGTH * 3 + GAP_SLOT,
                                                      startY - GAP_MODULE_LABEL);

            showModuleImages();
        }

    }
}

