﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace GetRotaryID
{
    public partial class Form1 : Form
    {
        const int ROTARY_SWITH_UNIT = 400;
        const int ARROW_UNIT = 120;
        const int CIRCLE_UNIT = 150;
        const int NUMBER_UNIT = 175;
        const int ORIGIN_X = ROTARY_SWITH_UNIT / 2;
        const int ORIGIN_Y = ROTARY_SWITH_UNIT / 2;

        //arrowPoints = vertices of the arrow shape
        double[] arrowPointsX = new double[] {1, 0, 0, -1, -1, 0, 0};
        double[] arrowPointsY = new double[] {0, -1, -0.5, -0.5, 0.5, 0.5, 1};
        const int ARROW_POINT_NUM = 7;

        Label[] lbNumbers = new Label[10]; //numbers of the rotary switch

        int CurrentRotaryID = 0;
        int previousRotaryID = 0;  //used to prevent flash

        public Form1()
        {
            InitializeComponent();
            double angle;
            int x, y;

            for (int i = 0; i < 10; i++) 
            {
                angle = Math.PI * 2 / 10 * i;

                //labels of number initialization
                lbNumbers[i] = new Label();
                x = Convert.ToInt32(Math.Cos(angle) * NUMBER_UNIT + ORIGIN_X);
                y = Convert.ToInt32(Math.Sin(angle) * NUMBER_UNIT + ORIGIN_Y);
                lbNumbers[i].Location = new Point(x,y);
                lbNumbers[i].Text = i.ToString();
                lbNumbers[i].Font = new Font(FontFamily.GenericSerif, 16, FontStyle.Bold);
                lbNumbers[i].ForeColor = Color.White;
                lbNumbers[i].BackColor = Color.Black;
                lbNumbers[i].Size = new Size(20, 20); 
                this.panel1.Controls.Add(lbNumbers[i]);
            }
        }

        private double[] rotate(double[] x, double[] y, int length, int graduation)
        {
            //vector rotation formula:
            // x2 = x1 * cos - y1 * sin
            //y2 = y1 * cos + x1 * sin

            double angle = Math.PI * 2 / 10 * graduation;
            ArraySegment<double> dbArray = new ArraySegment<double>(new double[length*2] , 0, length*2);

            for(int i=0;i<length; i++)
            {
                dbArray.Array[2 * i] = x[i] * Math.Cos(angle) - y[i] * Math.Sin(angle);
                dbArray.Array[2 * i + 1] = y[i] * Math.Cos(angle) + x[i] * Math.Sin(angle);
            }

            return dbArray.Array;
        }

        private Point[] CalculateArrowPoints(int graduation)
        {
            double[] rotatedArrowPoints = new double[ARROW_POINT_NUM * 2];
            Point[] rotatedArrow = new Point[ARROW_POINT_NUM];
            int x, y;

            rotatedArrowPoints = rotate(arrowPointsX, arrowPointsY, ARROW_POINT_NUM, graduation);
            
            for (int i = 0; i < ARROW_POINT_NUM; i++)
            {
                x = Convert.ToInt32(ARROW_UNIT * rotatedArrowPoints[2 * i] + ORIGIN_X);
                y = Convert.ToInt32(ARROW_UNIT * rotatedArrowPoints[2 * i + 1] + ORIGIN_Y);
                rotatedArrow[i] = new Point(x, y);
            }

            return rotatedArrow;
        }

        private void panel1_Paint(object sender, PaintEventArgs e)
        {
            Brush  BLACK = new SolidBrush(Color.Black);
            Brush WHITE = new SolidBrush(Color.White);
            Brush GRAY = new SolidBrush(Color.Gray);
            Point[] arrowPt = new Point[ARROW_POINT_NUM];

            e.Graphics.FillRectangle(BLACK, 0, 0, ROTARY_SWITH_UNIT, ROTARY_SWITH_UNIT);

            e.Graphics.FillEllipse(WHITE,
                ORIGIN_X - CIRCLE_UNIT,
                ORIGIN_Y - CIRCLE_UNIT,
                CIRCLE_UNIT * 2,
                CIRCLE_UNIT * 2);

            arrowPt = CalculateArrowPoints(CurrentRotaryID);
            e.Graphics.FillPolygon(GRAY, arrowPt);
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            previousRotaryID = CurrentRotaryID;
            CurrentRotaryID = PACNET.Sys.GetRotaryID();

            if(CurrentRotaryID != previousRotaryID)  //to prevent flash
                panel1.Refresh();
        }
    }
}