//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this sample source code is subject to the terms of the Microsoft
// license agreement under which you licensed this sample source code. If
// you did not accept the terms of the license agreement, you are not
// authorized to use this sample source code. For the terms of the license,
// please see the license agreement between you and Microsoft or, if applicable,
// see the LICENSE.RTF on your install media or the root of your tools installation.
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES.
//
/*
 *    Module Name:  
 *       oalintr.h
 *
 *    Abstract:
 *          System Interrupt definitions for OAL layer.  SYSINTR_FIRMWARE is
 *       defined in nkintr.h.  The OAL can use any values in the range
 *       SYSINTR_FIRMWARE thru SYSINTR_MAXIMUM.
 *
 */

// Note : The CEPC is a bit unusual since we allow various
// cards to be a different addresses without actually defining
// a new platform.  So, for those drivers there is no need
// to define an actual SYSINTR value.  If the driver is loaded
// by a bus driver, the SYSINTR value will be provided (in the
// registry) by the bus driver otherwise OALIntrRequestSysIntr may
// be used to request a SYSINTR <-> IRQ mapping.
//
// Define SYSINTR values for statically-mapped (SYSINTR <-> IRQ map) devices.
//
#define SYSINTR_KEYBOARD    (SYSINTR_FIRMWARE+1)
#define SYSINTR_TOUCH       (SYSINTR_FIRMWARE+12)

// And Audio hasn't been modified yet, but should be.
#define SYSINTR_AUDIO       (SYSINTR_FIRMWARE+4)

//=============================================================================
//@peter
//=============================================================================

#define SYSINTR_BP_BEGIN		    (SYSINTR_FIRMWARE  + 16)//32

#define SYSINTR_BP_COM3				(SYSINTR_BP_BEGIN  + 0) //32, 0x20
#define SYSINTR_BP_COM4	    		(SYSINTR_BP_BEGIN  + 1) //33, 0x21
#define SYSINTR_BP_COM5	    		(SYSINTR_BP_BEGIN  + 2) //34, 0x22

#define SYSINTR_BP_HOTPLUG	    	(SYSINTR_BP_BEGIN  + 3) //35
#define SYSINTR_BP_SLOT_1	    	(SYSINTR_BP_BEGIN  + 4) //36
#define SYSINTR_BP_SLOT_2	    	(SYSINTR_BP_SLOT_1 + 1) //37
#define SYSINTR_BP_SLOT_3	    	(SYSINTR_BP_SLOT_1 + 2)
#define SYSINTR_BP_SLOT_4	    	(SYSINTR_BP_SLOT_1 + 3)
#define SYSINTR_BP_SLOT_5	    	(SYSINTR_BP_SLOT_1 + 4)
#define SYSINTR_BP_SLOT_6	    	(SYSINTR_BP_SLOT_1 + 5)
#define SYSINTR_BP_SLOT_7	    	(SYSINTR_BP_SLOT_1 + 6) //42

//SYSINTR 43-71(28) reserved to 8114

#define SYSINTR_BP_TIMER_0			(SYSINTR_FIRMWARE+56) //72
#define SYSINTR_BP_TIMER_1			(SYSINTR_FIRMWARE+57) //73
#define SYSINTR_BP_TIMER_2			(SYSINTR_FIRMWARE+58) //74

//=============================================================================
//@end
//=============================================================================

