#ifndef __WINPACSDK_H
#define __WINPACSDK_H


#ifdef __cplusplus
extern "C" {
#endif


#ifdef WINPACSDK_EXPORTS
#define WINPAC_API extern "C" __declspec(dllexport) 
#else
#define WINPAC_API extern "C" __declspec(dllimport) 
#endif

#define WINPAC_PRIVATE_API WINPAC_API


#define WINPAC_SERAL_NUMBER_LENGTH		8


//=============================================================================
//  System Functions
//=============================================================================
WINPAC_API int	pac_GetModuleType(BYTE slot);
WINPAC_API int	pac_GetModuleName(BYTE slot, LPTSTR strName);
WINPAC_API int	pac_GetRotaryID();
WINPAC_API void pac_EnableLED(BOOL bFlag);
WINPAC_API void pac_GetSerialNumber(LPTSTR SerialNumber, int dwLength);
WINPAC_API BYTE pac_GetCPUVersion();
WINPAC_API void pac_GetOSVersion(LPTSTR os_version, int dwLength);
WINPAC_API void pac_GetEbootVersion(LPTSTR EbootVersion, int dwLength);
WINPAC_API void pac_GetSDKVersion(LPTSTR SDKVersion, int dwLength);
WINPAC_API void pac_GetMacAddress(BYTE LAN, LPTSTR MacAddr, int dwLength);

WINPAC_API void pac_ChangeSlot(BYTE slotNo);

//=============================================================================
// Backplane API
//=============================================================================

WINPAC_API int	pac_GetNetID();
WINPAC_API WORD pac_GetSlotCount();

WINPAC_API void pac_RegistryHotPlug(DWORD hWnd, DWORD msgID);
WINPAC_API void pac_UnregistryHotPlug(DWORD hWnd);

WINPAC_API void pac_EnableHotPlug(BOOL bEnable);
WINPAC_API BOOL pac_GetHotPlugStatus();
WINPAC_API void pac_ClearHotPlug(BOOL bEnable);
WINPAC_API BOOL pac_GetHotPlugIRQState();

WINPAC_API void pac_ResetSystemToDefault();
WINPAC_API BYTE pac_GetRisingEdgeIRQStatus();
WINPAC_API BYTE pac_GetRisingEdgeEnableStatus();
WINPAC_API void pac_ClearRisingEdgeIRQ(BYTE slots);
WINPAC_API void pac_EnableRisingEdgeIRQ(BYTE slots, BOOL bEnable);
WINPAC_API void pac_EOI();
WINPAC_API void pac_EnableRetrigger(BYTE iValues);
WINPAC_API void pac_ResetBackplane();
WINPAC_API int  pac_GetBackplaneID();

//=============================================================================
// Memory API
//=============================================================================
#define WINPAC_MEM_SRAM		0
#define WINPAC_MEM_EEPROM	1

WINPAC_API DWORD pac_GetMemorySize(int mem_type);
WINPAC_API int	 pac_ReadMemory(DWORD ndx, LPBYTE lpBuffer, DWORD dwLength, int mem_type);
WINPAC_API void	 pac_WriteMemory(DWORD ndx, LPBYTE lpBuffer, DWORD dwLength, int mem_type);
WINPAC_API void	 pac_ProtectEEPROM(void);
WINPAC_API void  pac_EnableEEPROM(void);


//=============================================================================
// Watch Dog Timer Functions
//=============================================================================
#define WINPAC_WDT_HW		0
#define WINPAC_WDT_OS		1

WINPAC_API void	 pac_EnableWatchDog(int wdt, DWORD value);
WINPAC_API void  pac_DisableWatchDog(int wdt);
WINPAC_API void  pac_RefreshWatchDog(int wdt);
WINPAC_API BOOL  pac_GetWatchDogState(int wdt);
WINPAC_API DWORD pac_GetWatchDogTime(int wdt);
WINPAC_API void  pac_SetWatchDogTime(int wdt, DWORD value);
WINPAC_API void  pac_Reboot();


//=============================================================================
// Micro SD Management API
//=============================================================================

WINPAC_API BOOL pac_SDExists();
WINPAC_API BOOL pac_SDMount(LPTSTR szPartitionName);
WINPAC_API BOOL pac_SDUnmount();
WINPAC_API BOOL pac_SDOnside();


//=============================================================================
// Registry API
//=============================================================================

#define PKT_NONE	0
#define PKT_KEY		1
#define PKT_STRING	2
#define PKT_DWORD	3

WINPAC_API int   pac_RegSave(LPCTSTR KeyName); 
WINPAC_API BOOL  pac_RegKeyExist(LPCTSTR KeyName);
WINPAC_API int   pac_RegDelete(LPCTSTR KeyName);
WINPAC_API int   pac_RegSetString(LPCTSTR KeyName, LPCTSTR assignStr, DWORD dwLength);
WINPAC_API int   pac_RegSetDWORD(LPCTSTR KeyName, DWORD assignVal);
WINPAC_API int   pac_RegCreateKey(LPCTSTR KeyName);
WINPAC_API DWORD pac_RegCountKey(LPCTSTR KeyName);
WINPAC_API DWORD pac_RegCountValue(LPCTSTR KeyName);
WINPAC_API int   pac_RegGetKeyIndex(LPCTSTR KeyName, DWORD dwIndex, LPTSTR lpName);
WINPAC_API int   pac_RegGetValueIndex(LPCTSTR KeyName, DWORD dwIndex, LPTSTR lpName);
WINPAC_API DWORD pac_RegGetKeyInfo(LPCTSTR KeyName);
WINPAC_API int   pac_RegGetString(LPCTSTR KeyName, LPTSTR lpData, DWORD dwLength);
WINPAC_API DWORD pac_RegGetDWORD(LPCTSTR KeyName);



//=============================================================================
// UART API
//=============================================================================

#define CTO_READ_TIMEOUT    	1
#define CTO_READ_TOTAL_TIMEOUT	2
#define CTO_WRITE_TOTAL_TIMEOUT	3


WINPAC_API HANDLE uart_Open(LPCSTR ConnectionString);
WINPAC_API BOOL   uart_Close(HANDLE hPort);
WINPAC_API BOOL   uart_Send(HANDLE hPort, LPCSTR buf);
WINPAC_API BOOL   uart_SendEx(HANDLE hPort, LPCTSTR buf);
WINPAC_API LPCSTR uart_Recv(HANDLE hPort);
WINPAC_API BOOL	  uart_RecvEx(HANDLE hPort, char *buf);

WINPAC_API LPCSTR uart_SendCmd(HANDLE hPort, LPCSTR cmd);
WINPAC_API BOOL uart_SendCmdEx(HANDLE hPort, LPCTSTR cmd, LPTSTR szResult);

WINPAC_API void   uart_SetTimeOut(HANDLE hPort, DWORD msec, int ctoType);
WINPAC_API void	  uart_EnableCheckSum(BOOL bEnable);  // Disable by default
WINPAC_API void	  uart_SetTerminator(LPCSTR szTerm); // CR by default
WINPAC_API DWORD  uart_GetLastError();


//=============================================================================
// Undocumented API
//=============================================================================


#define PAC_INP		_pac_inp
#define PAC_OUTP	_pac_outp
#define PAC_MAP		_pac_map


WINPAC_PRIVATE_API BYTE	 _pac_inp(int addr);
WINPAC_PRIVATE_API void	 _pac_outp(int addr, BYTE val);
WINPAC_PRIVATE_API PBYTE _pac_map(int addr);	
WINPAC_PRIVATE_API void	 _pac_refresh_screen();
WINPAC_PRIVATE_API BYTE  _pac_slot_inp(BYTE slot, int addr);
WINPAC_PRIVATE_API void  _pac_slot_outp(BYTE slot, int addr, BYTE val);
WINPAC_PRIVATE_API PBYTE _pac_slot_map(BYTE slot, int addr);
WINPAC_PRIVATE_API int	 _pac_get_slot_offset(BYTE slot);
WINPAC_PRIVATE_API PBYTE _pac_get_base_address();
WINPAC_PRIVATE_API void  _pac_set_mac_address(BYTE LAN, LPTSTR MacAddr, int dwLength);

//-----------------------------------------------------------------------------
// WINPAC Utility API
//-----------------------------------------------------------------------------
WINPAC_PRIVATE_API void _pac_alter_mermory_storage(int pos);
WINPAC_PRIVATE_API void _pac_get_memory_info(DWORD *PageSize, DWORD *TotPages, DWORD *StoreUsed, DWORD *RamUsed, DWORD *StorePages);
WINPAC_PRIVATE_API void _pac_get_program_info(DWORD PageSize, DWORD *pRamUsed);

WINPAC_PRIVATE_API BOOL _pac_get_partition_info(LPTSTR szPartName,LPTSTR szFileSys, LPTSTR szVolName);

#ifdef __cplusplus
}
#endif

#endif // __WINPACSDK_H
